<?php


namespace Tool\MayouTool\Mq;


use Tool\MayouTool\Bean\Mq\ConsumeParamBean;
use Tool\MayouTool\MqTool;

abstract class MqConsumeInterface
{
    /**
     * 交换机名称
     * @var string $exchangeName
     */
    protected $exchangeName;

    /**
     * 队列名称
     * @var string $queueName
     */
    protected $queueName;

    /**
     * 路由键
     * @var string $routeKey
     */
    protected $routeKey;

    /**
     * 交换机类型(默认直连模式)
     * @var string $exchangeType
     */
    protected $exchangeType = AMQP_EX_TYPE_DIRECT;

    /**
     * 消费前置动作
     * @return mixed
     */
    public function bootStart()
    {
    }

    /**
     * 消费后置动作
     * @return mixed
     */
    public function bootEnd()
    {
    }

    /**
     * 消费处理函数
     * @param \AMQPEnvelope $AMQPEnvelope
     * @param \AMQPQueue $AMQPQueue
     * @return mixed
     */
    public abstract function consumeCallback(\AMQPEnvelope $AMQPEnvelope, \AMQPQueue $AMQPQueue);

    public function handleConsume()
    {
        $params = [
            "exchangeName" => $this->exchangeName,
            "exchangeType" => $this->exchangeType,
            "queueName"    => $this->queueName,
            "routeKey"     => $this->routeKey,
            "callback"     => [ $this, "consumeCallback" ],
            "object"       => $this
        ];
        $consumeParamBean = new ConsumeParamBean($params);
        $rabbitMq = RabbitMq::getInstance();
        //声明交换机
        $rabbitMq->declareExchange($consumeParamBean->getExchangeName(), $consumeParamBean->getExchangeType());
        //声明队列
        $rabbitMq->declareQueue($consumeParamBean->getQueueName(), $consumeParamBean->getRouteKey());
        while (true) {
            try {
                MqTool::consume($consumeParamBean, $rabbitMq);
            } catch (\Throwable $exception) {
            }
        }
    }
}
