<?php


namespace Tool\MayouTool;


use Tool\MayouTool\Bean\Mq\ConsumeParamBean;
use Tool\MayouTool\Exception\MqException;
use Tool\MayouTool\Mq\RabbitMq;
use Tool\MayouTool\Mq\MqService;

class MqTool
{
    /**
     * 推送日志消息到mq
     * @param $message 消息
     * @param $routeKey 路由键
     */
    public static function pushLogMessage($message, $routeKey)
    {
        MqService::pushLogMessage($message, $routeKey);
    }

    /**
     * 推送消息的mq
     * @param $message 消息
     * @param $exchange 交换机
     * @param $routeKey 路由键
     */
    public static function pushMessage($message, $exchange, $routeKey)
    {
        MqService::pushMessage($message, $exchange, $routeKey);
    }

    /**
     * 消费消息队列
     * @param ConsumeParamBean $consumeParamBean
     * @throws MqException
     * @throws \AMQPChannelException
     * @throws \AMQPConnectionException
     */
    public static function consume(ConsumeParamBean $consumeParamBean, RabbitMq $rabbitMq)
    {
        if ($AMQPEnvelope = $rabbitMq->getQueue()->get()) {
            //消费消息
            $consumeParamBean->getObject()->bootStart();
            call_user_func_array($consumeParamBean->getCallback(), [ $AMQPEnvelope, $rabbitMq->getQueue() ]);
            $consumeParamBean->getObject()->bootEnd();
        } else {
            sleep(1);
        }
    }

    /**
     * 向交换机发布消息
     * @param mixed $exchangeName 交换机名称
     * @param mixed $message 消息
     * @param string $routeKey 路由键
     */
    public static function publish($message = '', $routeKey = '', $singleton = true)
    {
        $rabbitMq = RabbitMq::getInstance($singleton);
        //推送消息
        $rabbitMq->publish($message, $routeKey);
    }
}